#pragma once

#include <map>
#include <string>

#include "LabeledExprBaseVisitor.h"

class LabeledExprEvalVisitor : public LabeledExprBaseVisitor {
public:
    virtual antlrcpp::Any visitProg(LabeledExprParser::ProgContext* ctx) override
    {
        return visitChildren(ctx);
    }

    virtual antlrcpp::Any visitPrintExpr(LabeledExprParser::PrintExprContext* ctx) override
    {
        auto value = this->visit(ctx->expr());
        std::cout << value.as<int>() << std::endl;
        return value;
    }

    virtual antlrcpp::Any visitAssign(LabeledExprParser::AssignContext* ctx) override
    {
        std::string id = ctx->ID()->getText();
        auto value = this->visit(ctx->expr());
        _memory[id] = value.as<int>();
        return value;
    }

    virtual antlrcpp::Any visitBlank(LabeledExprParser::BlankContext* ctx) override
    {
        return visitChildren(ctx);
    }

    virtual antlrcpp::Any visitParens(LabeledExprParser::ParensContext* ctx) override
    {
        return this->visit(ctx->expr());
    }

    virtual antlrcpp::Any visitMulDiv(LabeledExprParser::MulDivContext* ctx) override
    {
        int left  = this->visit(ctx->expr(0));
        int right = this->visit(ctx->expr(1));
        if(ctx->op->getType() == LabeledExprLexer::MUL){
            return left * right;
        } 
        else {
            return left / right;
        }
    }

    virtual antlrcpp::Any visitAddSub(LabeledExprParser::AddSubContext* ctx) override
    {
        int left  = this->visit(ctx->expr(0));
        int right = this->visit(ctx->expr(1));
        if(ctx->op->getType() == LabeledExprLexer::ADD){
            return left + right;
        } 
        else {
            return left - right;
        }
    }

    virtual antlrcpp::Any visitId(LabeledExprParser::IdContext* ctx) override
    {
        std::string id = ctx->ID()->getText();
        if(_memory.count(id))
            return _memory[id];
        else 
            return 0;
    }

    virtual antlrcpp::Any visitInt(LabeledExprParser::IntContext* ctx) override
    {
        std::string id = ctx->INT()->getText();
        return std::stoi(id);
    }

private:
    std::map<std::string, int> _memory;
};